==> ymusk-0.95.zip: ymusk-0.95/Hooks.pm <==
use vars qw/ %tags /;

sub check_hooks
{
	$_ = shift;
	my @tags;

	foreach my  $h ( keys %hooks )
	{
		next unless /$h/;

		push @tags, $hooks{ $h }->( $_ );
	}

	return @tags, @_;
}

$hooks{ qr#(http|ftp)://[\w./~]*# } = \&Hooks::url;

sub Hooks::url
{
	my @tags;
    while( $_[0] =~ m#((http|ftp)://[\w./~]*(:\d+)?)#g )
    {
        create_url_tag( 'url', $output, $1 );  
        push @tags, [ pos($_[0])-length $1, pos $_[0], "url:$1" ];
    }
	return @tags;
}

$hooks{ qr/\w+\@[.\w]+/ } = \&Hooks::email;

sub Hooks::email
{
	my @tags;
    while( $_[0] =~ m#(\w+\@[.\w]+)#g )
    {
        create_url_tag( 'email', $output, $1 );  
        push @tags, [ pos($_[0])-length $1, pos $_[0], "email:$1" ];
    }
    return @tags;
}

$hooks{ qr/^Log info:/ } = \&Hooks::logstart;
$tags{ 'comment' } = { html => [ '<!-- ', ' -->' ], tag => {} };
$tags{ 'h1' } = { html => [ '<h1>', '</h1>' ], tag => {} };

sub Hooks::logstart
{
	return ( [ 0, 10, 'comment' ], [10, length $_[0], 'h1' ] );
}

$tags{ 'page' } 
	= { html => [ '<p class="page">', '</p>' ],
		tag  => {} };
$tags{ 'page-header' } 
	= { html => [ '<span class="page-header">', '</span>' ],
		tag  => { -foreground => '#cdd268793937'} };
$hooks{ qr/^(From afar,|[)\w(]+ pages:|Long distance to|You paged)/ } 
	= \&Hooks::page;

sub Hooks::page
{
	my $line = shift;
	my @tags;
	push @tags, [ 0, length $line, 'page' ];
	if( $line =~ /^From afar, ([)\w(]+)/g        or
	    $line =~ /^([)\w(]+) pages:/g            or
		$line =~ /^Long distance to ([)\w(]+):/g or
		$line =~ /^You paged (\(blind\) )?(\w+)/g           )
	{
		push @tags, [ 0, pos $line, 'page-header' ];
		if( $ycq_mw )
		{
			my $name = $+;
			$name =~ s/\(.*\)//;
			Ycq::add_page( $name, $line );
		}
	}

	return @tags;
}

$tags{ 'ymusk' } 
	= { html => [ '<p class="ymusk">', '</p>' ],
	    tag  => {} };
$tags{ 'ymusk-header' } 
	= { html => [ '<span class="ymusk-header">', '</span>' ],
	    tag  => { -foreground => 'red' } };

$hooks{ qr/^YMUSK:/ } = \&Hooks::ymusk;
sub Hooks::ymusk
{
	return ( [0, length $_[0], 'ymusk'], [ 0, 6, 'ymusk-header' ] );
}

$tags{ 'game' } 
	= { html => [ '<p class="game">', '</p>' ],
	    tag  => {} };
$tags{ 'game-header' } 
	= { html => [ '<span class="game-header">', '</span>' ],
	    tag  => { -foreground => 'red' } };
$hooks{ qr/^GAME:/ } = \&Hooks::game;
$hooks{ qr/^Huh\?  \(Type "help" for help\.\)/ } = \&Hooks::game;

sub Hooks::game
{
	return ( [0, length $_[0], 'game'] ,[ 0, 6, 'game-header' ]);
}

$hooks{ qr/^</ } = \&Hooks::channel;

$tags{ channel } 
	= { html => [ '<p class="channel">', '</p>' ],
		tag  => {} };
$tags{ 'channel-header' } 
	= { html => [ '<span class="channel-header">', '</span>' ],
	    tag  => { -foreground => 'red' } };

sub Hooks::channel
{
	my $line = shift;
	$line =~ /^(<[^>]*>)/;
	return ( [ 0, length $1, 'channel-header' ],
	         [ 0, length $line, 'channel' ] );
}

$hooks{ qr/^\s*[-^*=]+\s*$/ }  = \&Hooks::hr;
$tags{ 'hr' } = { html => [ '<hr><!-- ', ' -->' ], tag => {}  };
sub Hooks::hr
{
    return [ 0, length $_[0], 'hr' ];
}

sub parse_html_log 
{
	# ( $line, @tags ) = @_;
	my $line = shift ;

	return if $line=~/^\s*$/;

	chomp $line;
	my %t;
	
	foreach ( sort { $a->[0] <=> $b->[0] or $a->[2] cmp $b->[2] } @_ )
	{
		next unless $tags{ $_->[2] };

		$t{ $_->[0] } .= $tags{ $_->[2] }->{html}->[0];
		$t{ $_->[1] }  = $tags{ $_->[2] }->{html}->[1] . $t{ $_->[1] };
	}

	my $last;
	my $output;
	my $tidbit;
	foreach( sort { $a<=>$b } keys %t )
	{
		# must escape <channels> and the such :P
		$tidbit = (substr $line, $last, 	$_ - $last);
		$tidbit =~ s/</&lt;/g;
		$output .= $tidbit . $t{ $_ };
		$last = $_;
	}
	$tidbit = substr $line, $last;
	$tidbit =~ s/</&lt;/g;
	$output .= $tidbit;
	$output = "<p>$output</p>" unless $output =~ /^<p/;

	# prettily wrap that. 
	$output =~ s/(.{0,60})(\s|$)/$1\n/g;
	# and remove those escape characters
	$output =~ s/\e.*?m//g;

	return $output;
}


@ansi{ 1, 4, 5, 7, 8 } = qw/ bold underline blink reverse concealed /;

my $i = 0;
for( qw/ black red green yellow blue magenta cyan white / )
{
	$ansi{ 30 + $i } = $_;
	$ansi{ 40 + $i } = 'bg_'.$_;
	$i++;

	$tags{ $_ } = { html => [ "<font color=$_>", '</font>' ],
					tag => { -foreground => $_ } };
	$tags{ 'bg_'.$_ }
			= { html => [ "<span style='background-color: $_'>", '</span>' ],
				tag => { -background => $_ } };
}

$tags{ 'bold' }
	= { html => [ "<b>", "</b>" ],
	    tag  => { -relief => 'raised'  } };
$tags{ 'underline' }
	= { html => [ "<u>", "</u>" ],
	    tag  => { -underline => 1 } };
$tags{ 'concealed' }
	= { html => [ "<i>", "</i>" ],
	    tag  => { -underline => 1 } };
$tags{ 'blink' }
	= { html => [ "<blink>", "</blink>" ],
	    tag  => { -underline => 1 } };
$tags{ 'reverse' }
	= { html => [ "<i>", "</i>" ],
	    tag  => { -foreground => 'white', -background => 'black' } };

sub parse_ansi_line
{
my $line = shift;
my $parsed_line;
my @tags;

while( ( $pos = index $line, "\e[" ) > -1 )
{
	$line =~ s/\e\[([\d;]+)m//;

	foreach my $code ( split ';', $1 )
	{
		if( $code )
		{
			push @tags, [ $pos, 'end', $ansi{$code} ] if $ansi{$code};	
		}
		else		# it is '0' => clear
		{
			foreach( @tags )
			{
				$_->[1] = $pos if $_->[1] eq 'end';
			}
		}
	}
}

foreach( @tags ){ $_->[1] = length $line if $_->[1] eq 'end'; }

return ( $line, @tags );
}


sub create_url_tag
{
	my( $type, $widget, $email ) = @_;
	return unless $widget;
	$widget->tagConfigure( "$type:$email", -underline => 1, -foreground => 'blue' );
	if( ($type eq 'email')?$email_program:$url_program )
	{
		$widget->tagBind( "$type:$email", '<Any-Enter>', sub{ shift->tagConfigure( "$type:$email", -foreground => 'red' ) } );
		$widget->tagBind( "$type:$email", '<Any-Leave>', sub{ shift->tagConfigure( "$type:$email", -foreground => 'blue' ) } );
		if( $type eq 'email' )
		{
			$widget->tagBind( "$type:$email", 
	                  '<1>', 
					    sub{ local $email_program = $email_program;
						     $email_program =~ s/%/$email/;
							 `$email_program &` }  
					);
		}
		else
		{
			$widget->tagBind( "$type:$email",
                      '<1>',
                        sub{ local $url_program = $url_program;
                             $url_program =~ s/%/$email/;
                             `$url_program &` }
                    );
        }

	}	
}
'end of module Hooks.pm';

==> ymusk-0.98.zip: ymusk-0.98/Hooks.pm <==
use vars qw/ %tags /;

sub check_hooks
{
	$_ = shift;
	my @tags;

	foreach my  $h ( keys %hooks )
	{
		next unless /$h/;

		push @tags, $hooks{ $h }->( $_ );
	}

	return @tags, @_;
}

$hooks{ qr#(http|ftp)://[\w./~]*# } = \&Hooks::url;

sub Hooks::url
{
	my @tags;
    while( $_[0] =~ m#((http|ftp)://[\w./~]*(:\d+)?)#g )
    {
        create_url_tag( 'url', $output, $1 );  
        push @tags, [ pos($_[0])-length $1, pos $_[0], "url:$1" ];
    }
	return @tags;
}

$hooks{ qr/\w+\@[.\w]+/ } = \&Hooks::email;

sub Hooks::email
{
	my @tags;
    while( $_[0] =~ m#(\w+\@[.\w]+)#g )
    {
        create_url_tag( 'email', $output, $1 );  
        push @tags, [ pos($_[0])-length $1, pos $_[0], "email:$1" ];
    }
    return @tags;
}

$hooks{ qr/^Log info:/ } = \&Hooks::logstart;
$tags{ 'comment' } = { html => [ '<!-- ', ' -->' ], tag => { -foreground =>'#d915d915d915' } };
$tags{ 'h1' } = { html => [ '<h1>', '</h1>' ], tag => {} };

sub Hooks::logstart
{
	return ( [ 0, 10, 'comment' ], [10, length $_[0], 'h1' ] );
}

$tags{ 'page' } 
	= { html => [ '<span class="page">', "</span>\n" ],
		tag  => {} };
$tags{ 'page-header' } 
	= { html => [ '<span class="page-header">', '</span>' ],
		tag  => { -foreground => '#cdd268793937'} };
$hooks{ qr/^(From afar,|[)\w(]+ pages:|Long distance to|You paged)/ } 
	= \&Hooks::page;

sub Hooks::page
{
	my $line = shift;
	my @tags;
	if( $line =~ /^From afar, ([)\w(]+)/g        or
	    $line =~ /^([)\w(]+) pages:/g            or
		$line =~ /^Long distance to ([)\w(]+):/g or
		$line =~ /^You paged (\(blind\) )?(\w+)/g           )
	{
		push @tags, [ 0, pos $line, 'page-header' ];
		if( $ycq_mw )
		{
			my $name = $+;
			$name =~ s/\(.*\)//;
			Ycq::add_page( $name, $line );
		}
	}
	push @tags, [ 0, -1 + length $line, 'page' ];

	return @tags;
}

$tags{ 'ymusk' } 
	= { html => [ '<span class="ymusk">', "</span>\n" ],
	    tag  => {} };
$tags{ 'ymusk-header' } 
	= { html => [ '<span class="ymusk-header">', '</span>' ],
	    tag  => { -foreground => 'red' } };

$hooks{ qr/^YMUSK:/ } = \&Hooks::ymusk;
sub Hooks::ymusk
{
	return ( [0, -1 + length $_[0], 'ymusk'], [ 0, 6, 'ymusk-header' ] );
}

$tags{ 'game' } 
	= { html => [ '<span class="game">', "</span>\n" ],
	    tag  => {} };
$tags{ 'game-header' } 
	= { html => [ '<span class="game-header">', '</span>' ],
	    tag  => { -foreground => 'red' } };
$hooks{ qr/^GAME:/ } = \&Hooks::game;
$hooks{ qr/^Huh\?  \(Type "help" for help\.\)/ } = \&Hooks::game;

sub Hooks::game
{
	return ( [0, -1 + length $_[0], 'game'] ,[ 0, 6, 'game-header' ]);
}

$hooks{ qr/^</ } = \&Hooks::channel;

$tags{ channel } 
	= { html => [ '<span class="channel">', "</span>\n" ],
		tag  => {} };
$tags{ 'channel-header' } 
	= { html => [ '<span class="channel-header">', '</span>' ],
	    tag  => { -foreground => 'red' } };

sub Hooks::channel
{
	my $line = shift;
	$line =~ /^(<[^>]*>)/;
	return ( [ 0, length $1, 'channel-header' ] , [ 0, -1 + length $line, 'channel' ]);
}

$hooks{ qr/^\s*[-^*=]+\s*$/ }  = \&Hooks::hr;
$tags{ 'hr' } = { html => [ '<hr><!-- ', ' -->' ], tag => {}  };
sub Hooks::hr
{
    return [ 0, -1 + length $_[0], 'hr' ];
}

sub parse_html_log 
{
	# ( $line, @tags ) = @_;
	my $line = shift ;

	return if $line=~/^\s*$/;

	chomp $line;
	my %t;
	
	foreach ( sort { $a->[0] <=> $b->[0] or $a->[2] cmp $b->[2] } @_ )
	{
		next unless $tags{ $_->[2] };

		$t{ $_->[0] } .= $tags{ $_->[2] }->{html}->[0];
		$t{ $_->[1] }  = $tags{ $_->[2] }->{html}->[1] . $t{ $_->[1] };
	}

	my $last;
	my $output;
	my $tidbit;
	foreach( sort { $a<=>$b } keys %t )
	{
		# must escape <channels> and the such :P
		$tidbit = (substr $line, $last, 	$_ - $last);
		$tidbit =~ s/</&lt;/g;
		$output .= $tidbit . $t{ $_ };
		$last = $_;
	}
	$tidbit = substr $line, $last;
	$tidbit =~ s/</&lt;/g;
	$output .= $tidbit;
	$output = "<p>$output</p>" unless $output =~ /^<p/;

	# prettily wrap that. 
	$output =~ s/(.{0,60})(\s|$)/$1\n/g;
	# and remove those escape characters
	$output =~ s/\e.*?m//g;

	return $output;
}


@ansi{ 1, 4, 5, 7, 8 } = qw/ bold underline blink reverse concealed /;

my $i = 0;
for( qw/ black red green yellow blue magenta cyan white / )
{
	$ansi{ 30 + $i } = $_;
	$ansi{ 40 + $i } = 'bg_'.$_;
	$i++;

	$tags{ $_ } = { html => [ "<font color=$_>", '</font>' ],
					tag => { -foreground => $_ } };
	$tags{ 'bg_'.$_ }
			= { html => [ "<span style='background-color: $_'>", '</span>' ],
				tag => { -background => $_ } };
}

$tags{ 'bold' }
	= { html => [ "<b>", "</b>" ],
	    tag  => { -relief => 'raised'  } };
$tags{ 'underline' }
	= { html => [ "<u>", "</u>" ],
	    tag  => { -underline => 1 } };
$tags{ 'concealed' }
	= { html => [ "<i>", "</i>" ],
	    tag  => { -underline => 1 } };
$tags{ 'blink' }
	= { html => [ "<blink>", "</blink>" ],
	    tag  => { -underline => 1 } };
$tags{ 'reverse' }
	= { html => [ "<i>", "</i>" ],
	    tag  => { -foreground => 'white', -background => 'black' } };

sub parse_ansi_line
{
my $line = shift;
my $parsed_line;
my @tags;

while( ( $pos = index $line, "\e[" ) > -1 )
{
	$line =~ s/\e\[([\d;]+)m//;

	foreach my $code ( split ';', $1 )
	{
		if( $code )
		{
			push @tags, [ $pos, 'end', $ansi{$code} ] if $ansi{$code};	
		}
		else		# it is '0' => clear
		{
			foreach( @tags )
			{
				$_->[1] = $pos if $_->[1] eq 'end';
			}
		}
	}
}

foreach( @tags ){ $_->[1] = length $line if $_->[1] eq 'end'; }

return ( $line, @tags );
}


sub create_url_tag
{
	my( $type, $widget, $email ) = @_;
	return unless $widget;
	$widget->tagConfigure( "$type:$email", -underline => 1, -foreground => 'blue' );
	if( ($type eq 'email')?$email_program:$url_program )
	{
		$widget->tagBind( "$type:$email", '<Any-Enter>', sub{ shift->tagConfigure( "$type:$email", -foreground => 'red' ) } );
		$widget->tagBind( "$type:$email", '<Any-Leave>', sub{ shift->tagConfigure( "$type:$email", -foreground => 'blue' ) } );
		if( $type eq 'email' )
		{
			$widget->tagBind( "$type:$email", 
	                  '<1>', 
					    sub{ local $email_program = $email_program;
						     $email_program =~ s/%/$email/;
							 `$email_program &` }  
					);
		}
		else
		{
			$widget->tagBind( "$type:$email",
                      '<1>',
                        sub{ local $url_program = $url_program;
                             $url_program =~ s/%/$email/;
                             `$url_program &` }
                    );
        }

	}	
}
'end of module Hooks.pm';
